@@ -11,7 +11,6 @@ details. */
 
 #include <features.h>
 #include <cygwin/socket.h>
-#include <sys/time.h>
 
 #ifdef __cplusplus
 extern "C"
@@ -50,4 +49,8 @@ extern "C"
 };
 #endif
 
+#if __SSP_FORTIFY_LEVEL > 0
+#include <ssp/socket.h>
+#endif
+
 #endif /* _SYS_SOCKET_H */
