@@ -14,6 +14,7 @@ details. */
 #include <unistd.h>
 #include <sys/param.h>
 #include "pinfo.h"
+#include "psapi.h"
 #include "cygtls.h"
 #include "path.h"
 #include "fhandler.h"
@@ -28,9 +29,9 @@ add_timeval (struct timeval *tv1, struct timeval *tv2)
 {
   tv1->tv_sec += tv2->tv_sec;
   tv1->tv_usec += tv2->tv_usec;
-  if (tv1->tv_usec >= USPERSEC)
+  if (tv1->tv_usec >= 1000000)
     {
-      tv1->tv_usec -= USPERSEC;
+      tv1->tv_usec -= 1000000;
       tv1->tv_sec++;
     }
 }
@@ -181,7 +182,10 @@ getrlimit (int resource, struct rlimit *rlp)
 	  __get_rlimit_stack (rlp);
 	  break;
 	case RLIMIT_NOFILE:
-	  rlp->rlim_cur = rlp->rlim_max = OPEN_MAX;
+	  rlp->rlim_cur = getdtablesize ();
+	  if (rlp->rlim_cur < OPEN_MAX)
+	    rlp->rlim_cur = OPEN_MAX;
+	  rlp->rlim_max = OPEN_MAX_MAX;
 	  break;
 	case RLIMIT_CORE:
 	  rlp->rlim_cur = cygheap->rlim_core;
